;  __   ___   ___  _  _   ___   _  
; |  \ |   | |    | \/ | |   | | \ |
; |__/ |---| |    |    | |---| |  \|
; |    |   | |___ |    | |   | |   |
;
; written by blackbird
; copyright by namco

	processor f8

;------------;
; BIOS calls ;
;------------;

clrscrn:	equ $00d0					; uses r31
delay:		equ $008f
pushk:		equ $0107					; used to allow more subroutine stack space
popk:		equ $011e
drawchar:	equ $0679

;---------------;
; Program Entry ;
;---------------;

	org	$800

cartridgeStart:
	.byte	$55						; valid cart indicator
	nop							; unused byte

cartridgeEntry:
	lis	$0						; init the h/w
	outs	1
	outs	4
	outs	5
	outs	0
                
	lisu	4						; r32 = complement flag
	lisl	0
	lr	S, A
                
	li	$c6						; set to three color, grey background
	lr	3, A						; clear screen to grey
	pi	clrscrn						;

;-------------------;
; Main Program Loop ;
;-------------------;
				
mainloop:
	li	$c0
	lr	1, A
	li	$80
	lr	2, A
	li	>map0
	lr	Qu, A
	li	<map0
	lr	Ql, A
	pi	drawMap
	jmp	mainloop

;-------------------;
; Drawing Functions ;
;-------------------;

; plot out a single point on the screen
; uses three registers as "arguments"
; r1 = color
; r2 = x (to screen) (0-101)
; r3 = y (to screen) (0-57)

plot:
	; set the color using r1
	; colors: $40-red, $80=blue, $0=green, $c0=bkg
	lr	A, 1
	outs	1

	; set the column using r2
	lr	A, 2
	ai	$4
	com
	outs	4

	; set the row using r3
	lr	A, 3
	ai	$4
	com
	outs	5

	; transfer data to the screen memory

	li	$60
	outs	0
	li	$50
	outs	0
	lis	6						; a delay
_plotDelay:	
	ai	$ff
	bnz	_plotDelay

	pop							; return from the subroutine

; draw an 8x8 tile from a data pointer
; r1 = color 1
; r2 = color 2
; r3 = x (to screen)
; r4 = y (to screen)
; Q = 16-bit address

drawTile:
	; registers reference:
	; r1 = reserved (plot color)
	; r2 = reserved (plot x)
	; r3 = reserved (plot y)
	; r4 = color 1
	; r5 = color 2
	; r6 = x
	; r7 = y
	; r8 = loop1 (row)
	; r9 = loop2 (column)
	; r10 = bitmask
	; r11 = graphics byte
	; Q = tile address
	; K = return address

	; save the return address
	lr	K, P
	; load the 16-bit address into the DCI
	lr	DC, Q

	; rearrange the registers
	lr	A, 4
	lr	7, A
	lr	A, 3
	lr	6, A
	lr	A, 2
	lr	5, A
	lr	A, 1
	lr	4, A

	; start row loop
	li	8
	lr	8, A
_drawTileRow:
	; setup the bit mask
	li	%10000000					; we only want the left-most pixel
	lr	10, A						; store it in r10
	; and get the graphics byte
	lm							; load the pixel and increase the DC
	lr	11, A						; store it in r11

	; start column loop
	li	8
	lr	9, A
_drawTileColumn:

	; find out the x address
	lr	A, 9						; load the new offset
	com							; and subtract it from 8
	inc							; via 2's complement
	ai	8						; subtract A from 8, ignore carry
	as	6						; add the tile's real x offset
	lr	2, A						; save it for the subroutine call
	; and the y address
	lr	A, 8						; load the new offset
	com							; and subtract it from 8
	inc							; via 2's complement
	ai	8						; subtract A from 8, ignore carry
	as	7						; add the tile's real y offset
	lr	3, A						; save it for the subroutine call

	; initially load color 1
	lr	A, 4
	lr	1, A
	; see if the pixel is color 1 or 2
	lr	A, 11						; load the byte
	ns	10						; mask it with the bitmask
	bz	_drawTilePixel					; if the pixel's off, don't use color 1
	; conditionally load color 2
	lr	A, 5
	lr	1, A

_drawTilePixel:
	pi	plot

	; roll the bitmask
	lr	A, 10
	sr	1
	lr	10, A

_drawTileCheckLoops:
	; check column loop
	ds	9
	bnz	_drawTileColumn

	; check row loop
	ds	8
	bnz	_drawTileRow

	pk							; return from the subroutine

; draw a map from a map pointer
; r1 = color 1
; r2 = color 2
; Q = address of map

drawMap:
	; this function uses several reserve registers
	; because the functions called need the immediate ones
	; r17-r18 = return address
	; r19-r20 = map address
	; r21 = x counter
	; r22 = y counter
	; r23 = color 1
	; r24 = color 2

	; ISAR high byte is constant
	lisu	2

	; load the return address into memory
	lr	K, P
	; high byte: Ku -> r17
	lisl	1
	lr	A, Ku
	lr	S, A
	; low byte: Kl -> r18
	lisl	2
	lr	A, Kl
	lr	S, A

	; load the map address into memory
	; high byte: Qu -> r19
	lisl	3
	lr	A, Qu
	lr	S, A
	; low byte: Ql -> r20
	lisl	4
	lr	A, Ql
	lr	S, A

	; backup the colors
	; color 1
	lisu	2
	lisl	7
	lr	A, 1
	lr	S, A						; r23
	; color 2
	lisu	3
	lisl	0
	lr	A, 2
	lr	S, A						; r24

	lisu	2
	lisl	5						; 
	li	7						; load r17 with 12 tiles per row
	lr	S, A						;
_drawMapRow:

	lisl	6						;
	li	12						; load r18 with 7 rows of tiles
	lr	S, A						;
_drawMapColumn:

	; this is the start of drawing an individual tile

	; note of reference to drawTile:
	; r1 = color 1
	; r2 = color 2
	; r3 = x (to screen)
	; r4 = y (to screen)
	; Q = 16-bit tile address

	lisl	6						; load register r18
	lr	A, S
	lr	0, A						; set r0 as our row counter
	li	0						; clear A to the first pixel
_drawMapGetX:
	ds	0
	bz	_drawMapSaveX
	ai	8
	br	_drawMapGetX

_drawMapSaveX:
	com							; we want to display the tiles from
	inc							; top to bottom, so we inverse
	ai	88						; both coordinates
	lr	3, A						; store the x coordinate in r3

	lisl	5						; load register r17
	lr	A, S
	lr	0, A						; set r0 as our column counter
	li	0						; clear A to zero
_drawMapGetY:
	ds	0
	bz	_drawMapSaveY
	ai	8
	br	_drawMapGetY

_drawMapSaveY:
	com							; we want to display the tiles from
	inc							; top to bottom, so we inverse
	ai	47						; both coordinates
	lr	4, A						; store the y coordinate in r4

	; now we get the address of the tile

	; this is complex; first we take out the map address,
	; then we store it in the DC. We then get another byte,
	; and then do the reserve process. this needs to be optimized...
	; get the high byte
	lisu	2
	lisl	3						; r19
	lr	A, S
	lr	Qu, A
	; and the low byte
	lisl	4						; r20
	lr	A, S
	lr	Ql, A
	; transfer it into the DC
	lr	DC, Q
	; get the next tile byte
	lm
	; and back it up
	lr	0, A
	; transfer back the updated address
	lr	Q, DC
	; save the high byte
	lr	A, Qu
	lisl	3						; r19
	lr	S, A
	; and the low byte
	lr	A, Ql
	lisl	4						; r20
	lr	S, A

	; reset the DC to the tiles array
	li	>tiles
	lr	Qu, A
	li	<tiles
	lr	Ql, A
	lr	DC, Q

	; now we get the relative tile address
	lr	A, 0
	inc
	lr	0, A
	li	8						; load A with an increment of 8
_drawMapGetAddress:
	ds	0
	bz	_drawMapSaveAddress
	adc							; add 8 for every tile to the DC
	br	_drawMapGetAddress

_drawMapSaveAddress:
	lr	Q, DC						; transfer back our new address

_drawMapRestoreColors:
	; restore the colors
	; color 1
	lisu	2
	lisl	7
	lr	A, S						; r23
	lr	1, A						; into r1
	; color 2
	lisu	3
	lisl	0
	lr	A, S						; r24
	lr	2, A						; into r2

	; finally, call the tile function
	pi	drawTile

_drawMapCheckLoops:
	lisu	2
	; check column loop
	lisl	6
	ds	S
	bnz	_drawMapColumn

	; check row loop
	lisl	5
	ds	S
	bnz	_drawMapRow

_drawMapReturn:
	; return to the program flow
	; high byte: r17 -> Ku
	lisu	2
	lisl	1
	lr	A, S
	lr	Ku, A
	; low byte: r18 -> Kl
	lisu	2
	lisl	2
	lr	A, S
	lr	Kl, A

	pk							; return from the subroutine

;----------;
; Graphics ;
;----------;

	org $f00

tiles:
	; tile 0
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000

	; tile 1
	.byte	%00000000
	.byte	%00111111
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000

	; tile 2
	.byte	%00000000
	.byte	%11111111
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%10000000

	; tile 3
	.byte	%00000000
	.byte	%11111111
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%11111111

	; tile 4
	.byte	%00000000
	.byte	%11111110
	.byte	%00000001
	.byte	%00000001
	.byte	%00000001
	.byte	%00000001
	.byte	%00000001
	.byte	%10000001

	; tile 5
	.byte	%01000001
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%00111111

	; tile 6
	.byte	%01000001
	.byte	%10000001
	.byte	%00000001
	.byte	%00000001
	.byte	%00000001
	.byte	%00000001
	.byte	%00000001
	.byte	%10000001

	; tile 7
	.byte	%01000001
	.byte	%10000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%11111111

	; tile 8
	.byte	%01000001
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000
	.byte	%01000000

	; tile 9
	.byte	%01000001
	.byte	%01000001
	.byte	%01000001
	.byte	%01000001
	.byte	%01000001
	.byte	%01000001
	.byte	%01000001
	.byte	%01000001

	; tile 10
	.byte	%01000001
	.byte	%10000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%00000000
	.byte	%10000000

	; tile 11
	.byte	%01000001
	.byte	%10000001
	.byte	%00000001
	.byte	%00000001
	.byte	%00000001
	.byte	%00000001
	.byte	%00000001
	.byte	%11111110

;------------;
; Level Maps ;
;------------;

	org $f80

map0:
	.byte	0,  1,  2,  2,  3,  4,  1,  3,  2,  2,  4,  0
	.byte	0,  5,  6,  5,  2,  7,  7,  2, 11,  8, 11,  0
	.byte	0,  0,  9,  1, 10,  3,  3, 10,  4,  9,  0,  0
	.byte	3,  3, 10,  6,  9,  0,  0,  9,  8, 10,  3,  3
	.byte	0,  0,  9,  8, 10,  3,  3, 10,  6,  9,  0,  0
	.byte	0,  1,  7, 11,  5,  4,  1, 11,  5,  7,  4,  0
	.byte	0,  5,  3,  3,  3,  7,  7,  3,  3,  3, 11,  0

;-----------;
; Signature ;
;-----------;

	org $ff0

signature:
	.byte	"Blackbird 2004"
